@@ -48,6 +48,7 @@ use constant SMBGROUP 		=> '513';
 use constant SMBACCTFLAGS       => '[U]'; 
 use constant GECOS              => 'Ebox file sharing user '; 
 use constant USERGROUP          => 513;
+use constant DEFAULT_SHELL	=> '/bin/false';
 # Home path for users and groups
 use constant BASEPATH          => '/home/samba';
 use constant USERSPATH 	       => BASEPATH . '/users';
@@ -124,6 +125,8 @@ sub _userCommonLdapAttrs
 	       sambaHomeDrive       => SMBHOMEDRIVE,
 
 	       sambaAcctFlags       => SMBACCTFLAGS,
+
+	       loginShell            => _loginShell(),
 	      };
 
   return $attrs;
@@ -155,12 +158,12 @@ sub _addUserLdapAttrs
 
 				     sambaHomePath        => _smbHomes() . $user,
 
-				     sambaProfilePath     => _smbProfiles . $user,
 				     sambaPrimaryGroupSID => $sid . '-' . SMBGROUP,
 				     sambaLMPassword      => $lm,
 				     sambaNTPassword      => $nt,
 
 				     sambaSID             => $sambaSID,
+				    
 
 				     # gecos                => GECOS
 				    ],
@@ -302,20 +305,35 @@ sub _addGroup ($$)
 	 
 	return unless ($self->{samba}->configured());
 
-	my $ldap  = $self->{ldap};
-	my $users = EBox::Global->modInstance('users');
-	
-	my $rid = 2 * $users->lastGid + 1001;
+	$self->addGroupLdapAttrs($group);
+}
 
-	my $sid = alwaysGetSID();
-	my $sambaSID = $sid . '-' .  $rid;
 
-	my $dn = "cn=$group," .  $users->groupsDn;
+
+sub addGroupLdapAttrs
+{
+    my ($self, $group, %params) = @_;
+
+    my $sambaSID = delete $params{SID};
+
+    my $ldap  = $self->{ldap};
+    my $users = EBox::Global->modInstance('users');
+    
+    my $rid = 2 * $users->lastGid + 1001;
+
+    if (not defined $sambaSID) {
+	my $baseSid = alwaysGetSID();
+	$sambaSID = $baseSid . '-' .  $rid;
+    }
+
+
+
+    my $dn = "cn=$group," .  $users->groupsDn;
         
-	unless ($self->_isSambaObject('sambaGroupMapping', $dn)) {
-	     my %attrs = (
+    unless ($self->_isSambaObject('sambaGroupMapping', $dn)) {
+	my %attrs = (
 		     changes => [
-                    		add => [
+				 add => [
                        		 	 objectClass    => 'sambaGroupMapping',
 					 objectClass    => 'eboxGroup',
                         		 sambaSID       => $sambaSID,
@@ -323,9 +341,62 @@ sub _addGroup ($$)
                         		 displayName    => $group,
                            		]
                   		]
-		         );
-	     $ldap->modify($dn, \%attrs);
-	}
+		    );
+	$ldap->modify($dn, \%attrs);
+    }
+}
+
+
+
+sub setUserSID
+{
+    my ($self, $user, $sid) = @_;
+    checkSID($sid) or
+	throw EBox::Exceptions::External(__('Incorrect SID: {s}', 's' => $sid));
+
+    my $users = EBox::Global->modInstance('users');
+    $users->userExists($user)  or
+	throw EBox::Exceptions::External(__x('User {u} does not exists', 'u' => $user));
+
+    my $dn =  $users->userDn($user);
+    $self->_isSambaObject('sambaSamAccount', $dn) or
+	throw EBox::Exceptions::External(__x('User {u} is not a samba user', 'u' => $user));
+    
+    $self->_setSID($dn, $sid);
+}
+
+
+sub setGroupSID
+{
+    my ($self, $group, $sid) = @_;
+    checkSID($sid) or
+	throw EBox::Exceptions::External(__('Incoorect SID: {s}', 's' => $sid));
+
+    my $users = EBox::Global->modInstance('users');
+    $users->groupExists($group)  or
+	throw EBox::Exceptions::External(__x('Group {g} does not exists', 'g' => $group));
+
+    my $dn = $users->groupDn($group);
+    $self->_isSambaObject('sambaGroupMapping', $dn) or
+	throw EBox::Exceptions::External(__x('Group {g} is not a samba group', 'g' => $group));
+    
+    $self->_setSID($dn, $sid);
+}
+
+
+sub _setSID
+{
+    my ($self, $dn, $sambaSID) = @_;
+
+	my %attrs = (
+		     changes => [
+				 replace => [
+                        		 sambaSID       => $sambaSID,
+                           		]
+                  		]
+		    );
+    $self->{ldap}->modify($dn, \%attrs);
+
 }
 
 sub _delGroup($$){
@@ -550,6 +621,13 @@ sub checkDomainSID
   return 1;
 }
 
+
+sub checkSID
+{
+    # we do not discrimintae now etween domainSID and a element SID
+    return checkDomainSID(@_);
+}
+
 sub getSID 
 {
   my $samba  = EBox::Global->modInstance('samba');
@@ -877,6 +955,7 @@ sub sambaDomainName
 	my %attrs = ( 
 			base => "dc=ebox", 
 			filter => "(objectclass=sambaDomain)", 
+			
 			scope => "sub"
 	    	    ); 
 	my $entry = $ldap->search(\%attrs)->pop_entry();
@@ -908,6 +987,7 @@ sub setSambaDomainName
 	my $ldap = $self->{ldap}; 
 	my $sid  = getSID();
 
+	my %domainAttrs = %{$self->_fetchDomainAttrs($domain)};
 	$self->deleteSambaDomainNameAttrs();
 	$self->deleteSambaDomains();
 
@@ -919,14 +999,40 @@ sub setSambaDomainName
 			'uidNumber'		=> $users->lastUid,
 			'gidNumber'		=> $users->lastGid,
 			'objectClass'		=> ['sambaDomain', 
-						    'sambaUnixidPool']
+						    'sambaUnixidPool'],
+			%domainAttrs
 			]
-		   );
+		);
 
 	my $dn = "sambaDomainName=$domain,dc=ebox";
 	$ldap->add($dn, \%attrs);
 }
 
+sub _fetchDomainAttrs
+{
+	my ($self, $domain) = @_;
+
+	my $ldap = EBox::Ldap->instance();
+
+	my @attrs = qw/sambaPwdHistoryLength sambaMaxPwdAge sambaLockoutThreshold/;
+	my $result = $ldap->search( 
+			{
+			base => $ldap->dn(),
+			filter => "sambaDomainName=$domain",
+			scope => 'sub',
+			attrs => [@attrs],
+			}
+			);
+
+	my $entry = $result->pop_entry();
+	return {} unless defined($entry);
+	my %attributes;
+	for my $attr (@attrs) {
+		$attributes{$attr} = $entry->get_value($attr);
+		return {} unless defined($attributes{$attr});
+	}
+	return \%attributes;
+}
 
 sub deleteSambaDomainNameAttrs
 {
@@ -1080,12 +1186,81 @@ sub updateNetbiosName
 		my $username = $user->{'username'};
 		my $dn = "uid=$username," .  $users->usersDn;
 		$ldap->modifyAttribute($dn, 'sambaHomePath', 
-					"\\\\$netbios\\homes\\$username");
+				"\\\\$netbios\\homes\\$username");
+		# XXX Check if we have to add or not, instead of
+		# add + [ delete ]
 		$ldap->modifyAttribute($dn, 'sambaProfilePath', 
-					"\\\\$netbios\\profiles\\$username");
+				"\\\\$netbios\\profiles\\$username");
+		unless ($self->{samba}->roamingProfiles()) {
+			my %attrs = ( changes => [ delete =>
+					[ sambaProfilePath => [] ]]);
+			$ldap->modify($dn, \%attrs);
+		}
+
 	}
 }
 
+# Method: updateSIDEntries
+#	
+#		Check and correct if there's any user or group with a wrong SID. Note
+#		that depending on when the user/group is created the SID might change.
+#		This method should be run in regenConfig
+#
+#	
+sub updateSIDEntries
+{
+	my ($self) = @_;
+
+	my $users = EBox::Global->modInstance('users');
+	my $ldap = $self->{'ldap'};
+	my $userDN = $users->usersDn();
+	my $sid = uc(getSID());
+	$sid = uc($sid);
+
+	my %attrs = (
+			base   => $userDN,
+			filter => "(&(objectclass=sambaSamAccount)(!(sambaSID=$sid*)))",
+			attrs  => ['sambaSID', 'sambaPrimaryGroupSID', 'dn'],
+			scope  => 'sub'
+			);
+
+	my $result = $ldap->search(\%attrs);
+
+	for my $entry ($result->entries()) {
+		my $oldSID = $entry->get_value('sambaSID');
+		my $oldGroupSID = $entry->get_value('sambaPrimaryGroupSID');
+		my ($lastNumbers) = $oldSID =~ /.*-(\d+)$/;
+		my $newSID = "$sid-$lastNumbers";
+		my ($lastNumbersGroup) = $oldGroupSID =~ /.*-(\d+)$/;
+		my $newGroupSID = "$sid-$lastNumbersGroup";
+		$ldap->modifyAttribute($entry->dn(), 'sambaSID', $newSID);
+		$ldap->modifyAttribute($entry->dn(), 
+				'sambaPrimaryGroupSID', 
+				$newGroupSID);
+	}
+
+	my $groupDN = $users->groupsDn();
+	%attrs = (
+			base   => $groupDN,
+			filter => "(&(objectclass=sambaGroupMapping)(!(sambaSID=$sid*)))",
+			attrs  => ['sambaSID', 'cn'],
+			scope  => 'sub'
+			);
+
+	$result = $ldap->search(\%attrs);
+	my %groupsToSkip = ('Administrators' => 1, 'Account Operators' => 1,
+                            'Print Operators' => 1, 'Backup Operators' => 1, 'Replicators' => 1);
+
+	for my $entry ($result->entries()) {
+		next if $groupsToSkip{$entry->get_value('cn')};
+		my $oldSID = $entry->get_value('sambaSID');
+		my ($lastNumbers) = $oldSID =~ /.*-(\d+)$/;
+		my $newSID = "$sid-$lastNumbers";
+		$ldap->modifyAttribute($entry->dn(), 'sambaSID', $newSID);
+	}
+}
+
+
 sub _isSambaObject($$$) {
 	my $self = shift;
 	my $object = shift;
@@ -1120,7 +1295,7 @@ sub sharedDirectories
   } @{ $self->groupShareDirectories() };
 
   my $users = EBox::Global->modInstance('users');
-  defined $users or throw EBox::Exceptions::Internal('Can not get users and groups module');
+  defined $users or throw EBox::Exceptions::Internal('Cannot get users and groups module');
 
   my @homedirs = map {  $_->{homeDirectory}} $users->users();
   push @dirs, @homedirs;
@@ -1145,4 +1320,14 @@ sub groupsPath
   return GROUPSPATH;
 }
 
+sub _loginShell
+{
+	my $shell = EBox::Config::configkey('login_shell');
+
+	if (defined($shell)) {
+		return $shell;
+	} else {
+		return DEFAULT_SHELL;
+	}
+}
 1;
